
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "ixpio.h"
// #include "mstep3.h"
#include "msw-mstep3.h"

//*********************************************************
#define LINE_SIZE  128
#define NAME_SIZE  16

#define YES    1
#define NO     0

#define ON     1
#define OFF    0

#define CW_CCW    0
#define PULSE_DIR 1

#define NORMAL_DIR   0
#define REVERSE_DIR  1

#define FW      0
#define BW      1
#define CW      0
#define CCW     1

#define X_axis   1
#define Y_axis   2
#define Z_axis   3

#define XY_plane 1
#define XZ_plane 2
#define YZ_plane 3

#define DDA_CW_CCW     0x00
#define DDA_DIR_PULSE  0x01
#define SERVO_ON       0x02
#define DDA_OE         0x08

#define ENC_MARK       0x30
#define ENC_AB_PHASE   0x00
#define ENC_CW_CCW     0x10
#define ENC_DIR_PULSE  0x20
#define ENC_EXTERNAL   0x40

#define READY 0
#define BUSY  1

#define pi 3.141592654
//*********************************************************
//-------------- hardware -------------------------	define in msw.h
/*#define RDFF2  0xc0    //R
#define MSC    0xc4    //R
#define DI     0xc8    //R
#define WRFF1  0xc0    //W
#define RSTFF1 0xc4    //W
#define DO     0xc8    //W
*/
//-----for tiger's register-------------------		define in msw.h
/*#define TJ_CNTL         0x00  //set reset and others
#define TJ_AUXC         0x02  //AUX pin control register 1:output
#define TJ_AUXD         0x03  //AUX pin data register
#define TJ_MASK         0x05  //interrupt mask
#define TJ_AUX_STATUS   0x07  //AUX pin status
#define TJ_POLARITY     0x2a  //AUX pin polarity
*/
//----state----------------------------------
//#define PHASE1          1		define in msw.h
#define PHASE2          2
#define PHASE3          3

//#define sIDLE           0		define in msw.h
#define sXYZ            1
#define sLINE           2
#define sCIRCLE         3
#define sARC            4
//----- synchoronus pattern -----------------
#define sync_pattern1 0x55 //0101 0101
#define sync_pattern2 0xAA //1010 1010

//----- command sets --------------
  //-- motion command
  #define BACK_HOME        0x01
  #define BACK_HOME01      0x02
  #define PULSE_MOVE       0x03
  #define CONSTANT_SPEED   0x04
  //-- interpolation command
  #define INTP_PULSE       0x10

  //-- set variable
  #define SET_CONTROL_MODE 0x21
  #define SET_VAR          0x22
  #define SET_DEFDIR       0x23
  #define SET_SERVO_ON     0x24
  #define SET_ZERO         0x2A
  #define SET_NONZERO      0x2B
  #define SET_NORMAL_CLOSE 0x2C

  //-- stop, break command
  #define RESET_SYSTEM     0x30
  #define STOP_AXIS        0x31
  #define STOP_ALL         0x32
  #define DEC_STOP         0x33

  //-- testing command
  #define EXIST            0x40

  //*********************************************************
      //----- (X,Y STATE) sub command set -----------
      #define M_BACK_HOME_S1         1
      #define M_BACK_HOME_S2         2
      #define M_BACK_HOME01          3
      #define M_PULSE_MOVE           4
      #define M_INTP_PULSE           5
      #define M_DEC_STOP             6
      #define M_CONSTANT_SPEED       7

      #define BREAK                  98
      #define STOP                   99

//----- for receive data ---------------------
unsigned char pc_byte[32];
//****************************************************************
//******************** for interpolation ****************
#define PROFILE_STOP 1
#define PROFILE_BUSY 2

#define d_DDA_cycle    10     //(1~255)
#define d_Acc_Dec      2     //(1~255)
#define d_Low_Speed    2
#define d_High_Speed   100
#define d_arc_speed    30


//********************************************************
//------ card parameter -----------------------------------	define in msw.h
/*typedef struct
  {
         unsigned int  base;
         unsigned char exist;
         unsigned char op;
         unsigned char workNo;
         unsigned char sync1;
         unsigned char sync2;

         unsigned char sys;
         unsigned char limit;
         unsigned char p1;
         unsigned char InWorkNo;
         unsigned char X_state;
         unsigned char Y_state;
	 unsigned char Z_state;
         long          command_X;
         long          position_X;
         long          command_Y;
         long          position_Y;
         long          command_Z;
         long          position_Z;
  }cardtype;
*/
cardtype card[16];

//------ interpolation system parameter --------------------	define in msw.h
/*typedef struct
{
   unsigned char command;
   unsigned char cardNo;
   unsigned char plane;
   long          x;
   long          y;
   long          z;
   long          R;
   unsigned char dir;
   unsigned int  speed;
   unsigned char acc_mode;
}system_type;
*/
system_type sys;

//------ for interpolation use ----------------------------
unsigned char DDA_cycle   =d_DDA_cycle;
unsigned char Acc_Dec     =d_Acc_Dec;
unsigned int  Low_Speed   =d_Low_Speed;
unsigned int  High_Speed  =d_High_Speed;
unsigned int  arc_speed   =d_arc_speed;
unsigned int  temp_speed=0;
unsigned char fifo_full=0;

//------ for interpolation command ------------------------
unsigned long total_pulseX=0;
unsigned long total_pulseY=0;
unsigned long total_pulseZ=0;
unsigned int  dx=0;
unsigned int  dy=0;
unsigned int  dz=0;
char          x_dir=1;
char          y_dir=1;
char          z_dir=1;
unsigned long total_pulse=0;
unsigned long rem_pulse=0;
unsigned int  d_length=0;
unsigned char Dec_Flag=0;
unsigned char profile_state=0;
unsigned char WorkFlag;

unsigned long pulse_R=0;
unsigned long pulse_R1=0;
unsigned long pulse_R2=0;
unsigned char MaxAxis=0;
double        total_length;

double   radius;
double   theta,d_theta;
double   Xn,Yn;
double   Xr,Yr;

double   th1,th2,th;
double   xx1,yy1;
double   xc,yc;
double   x_new,y_new;
double   x_old,y_old;

//unsigned char 
unsigned char state=PHASE1;


//####################################################################
void outportb_fifo(unsigned char cardNo, unsigned char cdata)
{
  //if FIFO1./FF = 0, waiting for /FF=1
  while ( (inportb(card[cardNo].base + MSC) & 0x02)==0) //fifo full
  {
     fifo_full=YES;
  };
  while ( (inportb(card[cardNo].base + MSC) & 0x02)==0) //fifo full
  {
     fifo_full=YES;
  };

  if (fifo_full==YES) delay(5);
  
  outportb(card[cardNo].base + WRFF1, cdata);
}
//----------------------------------------------------
void output_fifo(unsigned char cardNo,
                 unsigned char com_byte0,
                 unsigned char com_byte1,
                 unsigned char com_byte2,
                 unsigned char com_byte3,
                 unsigned char com_byte4,
                 unsigned char com_byte5,
                 unsigned char com_byte6,
                 unsigned char com_byte7,
                 unsigned char com_byte8)

{
unsigned char   parity;


  fifo_full=NO;
  if (card[cardNo].workNo<0x50) card[cardNo].workNo++;
  else                          card[cardNo].workNo=1;

  parity=  com_byte0 ^ com_byte1 ^ com_byte2
         ^ com_byte3 ^ com_byte4 ^ com_byte5
         ^ com_byte6 ^ com_byte7 ^ com_byte8
         ^ card[cardNo].workNo;
  
  outportb_fifo(cardNo,sync_pattern1);
  outportb_fifo(cardNo,sync_pattern2);
  outportb_fifo(cardNo,com_byte0);
  outportb_fifo(cardNo,com_byte1);
  outportb_fifo(cardNo,com_byte2);
  outportb_fifo(cardNo,com_byte3);
  outportb_fifo(cardNo,com_byte4);
  outportb_fifo(cardNo,com_byte5);
  outportb_fifo(cardNo,com_byte6);
  outportb_fifo(cardNo,com_byte7);
  outportb_fifo(cardNo,com_byte8);
  outportb_fifo(cardNo,card[cardNo].workNo);
  outportb_fifo(cardNo,parity);

  outportb_fifo(cardNo,sync_pattern1);
  outportb_fifo(cardNo,sync_pattern2);
  outportb_fifo(cardNo,com_byte0);
  outportb_fifo(cardNo,com_byte1);
  outportb_fifo(cardNo,com_byte2);
  outportb_fifo(cardNo,com_byte3);
  outportb_fifo(cardNo,com_byte4);
  outportb_fifo(cardNo,com_byte5);
  outportb_fifo(cardNo,com_byte6);
  outportb_fifo(cardNo,com_byte7);
  outportb_fifo(cardNo,com_byte8);
  outportb_fifo(cardNo,card[cardNo].workNo);
  outportb_fifo(cardNo,(unsigned char)(~parity));

}


//----------------------------------------------------
void MSTEP3_reset_fifo(unsigned char cardNo)
{
  //1. cancel all of command pending in the fifo
  outportb(card[cardNo].base + RSTFF1,0x00);

  //2. output a null function to complete previous command,
  //   if there is a residue command didn't accept
  output_fifo(cardNo,0,0,0,0,0,0,0,0,0);
}
//####################################################################
unsigned char MSTEP3_EXIST(unsigned char cardNo)
{
unsigned char ff2_empt;
unsigned char s1,s2,s3;
unsigned int  timeout;
  disable();
  MSTEP3_reset_fifo(cardNo);
  output_fifo(cardNo,EXIST,0x80,0x37,0x46,0,0,0,0,0);
  
  //waiting for response
  timeout=500;
  s1=0; s2=0; s3=0;
  delay(20);
  do
  {
         ff2_empt = (0x04 & inportb(card[cardNo].base + MSC));
         if (ff2_empt==0x04)
         {
                s3=inportb(card[cardNo].base + RDFF2);
                //cprintf("s1=%x, s2=%x, s3=%x\n",s1, s2, s3);
                if ((s1==0x80) && (s2==0x37) && (s3==0x46))
                {
                  enable();
                  return YES; //-------> return
                }
                s1=s2; s2=s3;
         };
	delay(2);
	 timeout--;
  } while (timeout>0);
  enable();
  return NO;        //-------> return
}
/* FIXME - Rewrite in msw.c */
//-----------------------------------------------------------------------
/*unsigned char CALLBACK MSTEP3_REGISTRATION(unsigned char cardNo, unsigned int address)
{
unsigned char j,GetCard;
unsigned int  wBoards,wRetVal;
//PCI parameter
unsigned int  wBase;
unsigned int  wIrq,wSubVendor,wSubDevice,wSubAux;
unsigned int  wSlotBus,wSlotDevice,wHardwareID;

  wRetVal=PIO_DriverInit(&wBoards,0x81,0x04,0x10);
  if (wBoards==0) return NO;

  GetCard=0;
  for (j=0; j<wBoards; j++)
  {
    PIO_GetConfigAddressSpace(j,&wBase, &wIrq, &wSubVendor, &wSubDevice,
                              &wSubAux, &wSlotBus, &wSlotDevice, &wHardwareID);
    if (wHardwareID==address)
     {
       GetCard=1;
       card[cardNo].base=wBase;
       card[cardNo].op=0;
       card[cardNo].workNo=1;
       card[cardNo].sync1=0;
       card[cardNo].sync2=0;
      //reset PCI card
       outport(card[cardNo].base + TJ_CNTL, 0x00); //set /reset=0
       outport(card[cardNo].base + TJ_CNTL, 0x01); //set /reset=1
       outport(card[cardNo].base + TJ_AUXC, 0x00); //set all AUX are input
     };
  };

  if (GetCard==0) return NO;

  card[cardNo].exist=MSTEP3_EXIST(cardNo);
  WorkFlag   =NO;
  state      =PHASE1;  
  sys.command=sIDLE;
  return card[cardNo].exist;
}
*/
//-----------------------------------------------------------------------
/*unsigned char CALLBACK MSTEP3_REGISTRATION1(unsigned char cardNo, unsigned int address)
{
unsigned char j,GetCard;
unsigned int  wBoards,wRetVal;
//PCI parameter
unsigned int  wBase;
unsigned int  wIrq,wSubVendor,wSubDevice,wSubAux;
unsigned int  wSlotBus,wSlotDevice,wHardwareID;

  wRetVal=PIO_DriverInit(&wBoards,0x81,0x04,0x10);
  cprintf("wBoards=%d\r\n",wBoards); //for testing
  if (wBoards==0) return NO;

  GetCard=0;
  for (j=0; j<wBoards; j++)
  {
    PIO_GetConfigAddressSpace(j,&wBase, &wIrq, &wSubVendor, &wSubDevice,
                              &wSubAux, &wSlotBus, &wSlotDevice, &wHardwareID);
    if (wHardwareID==address)
     {
       GetCard=1;
       card[cardNo].base=wBase;
       card[cardNo].base=wBase;
       card[cardNo].op=0;
       card[cardNo].workNo=2;
       card[cardNo].sync1=0;
       card[cardNo].sync2=0;
      //reset PCI card
       outport(card[cardNo].base + TJ_CNTL, 0x00); //set /reset=0
       outport(card[cardNo].base + TJ_CNTL, 0x01); //set /reset=1
       outport(card[cardNo].base + TJ_AUXC, 0x00); //set all AUX are input

       cprintf("wBase=%x h\r\n",wBase); //for testing
       cprintf("wIrq=%d\r\n",wIrq); //for testing
       cprintf("wSubVendeor=%x h\r\n",wSubVendor); //for testing
       cprintf("wSubDevice=%x h\r\n",wSubDevice); //for testing
       cprintf("wHardwareID=%d\r\n",wHardwareID); //for testing
     }
  }

  if (GetCard==0) return NO;

  card[cardNo].exist=MSTEP3_EXIST(cardNo);
  WorkFlag   =NO;
  state      =PHASE1;
  sys.command=sIDLE;  
  cprintf("Base=%x exist=%d\r\n",card[cardNo].base,card[cardNo].exist); //for testing
  return card[cardNo].exist;
}
*/
//----------------------------------------------------
unsigned char CALLBACK MSTEP3_DI(unsigned char cardNo)
{
  return inportb(card[cardNo].base + DI);
}
//----------------------------------------------------
void CALLBACK MSTEP3_DO(unsigned char cardNo, unsigned char value)
{
 disable();
  card[cardNo].op = value & 0x7F;
  outportb(card[cardNo].base + DO, card[cardNo].op);
 enable();
}
//----------------------------------------------------
void CALLBACK MSTEP3_DO1(unsigned char cardNo, unsigned char value)
{
 disable();
  card[cardNo].op = value;
  outportb(card[cardNo].base + DO, card[cardNo].op);
 enable();
}
//----------------------------------------------------
unsigned char CALLBACK MSTEP3_MSC(unsigned char cardNo)
{
  return inportb(card[cardNo].base + MSC);
}
//----------------------------------------------------
void CALLBACK MSTEP3_WAIT_X(unsigned char cardNo)
{
 if (card[cardNo].exist==NO) return;
 delay(50);
 // check /FIFO1=empty
 while ((inportb(card[cardNo].base + MSC) & 0x01)==0x01) {};

 //wait /XSTOP
 while ((inportb(card[cardNo].base + MSC) & 0x10)==0x10) {};
 delay(10);
 while ((inportb(card[cardNo].base + MSC) & 0x10)==0x10) {};
}                     
//----------------------------------------------------
unsigned char CALLBACK MSTEP3_IS_X_STOP(unsigned char cardNo)
{
 if (card[cardNo].exist==NO) return NO;
 delay(50);
 // check FIFO1=empty
 if ((inportb(card[cardNo].base + MSC) & 0x01)==0x01) return NO;

 //check /XSTOP
 if ((inportb(card[cardNo].base + MSC) & 0x10)==0x10) return NO;
 return YES;
}
//----------------------------------------------------
void CALLBACK MSTEP3_WAIT_Y(unsigned char cardNo)
{
 if (card[cardNo].exist==NO) return;
 delay(50);
 // check /FIFO1=empty
 while ((inportb(card[cardNo].base + MSC) & 0x01)==0x01) {};

 //wait /YSTOP
 while ((inportb(card[cardNo].base + MSC) & 0x20)==0x20) {};
 delay(10);
 while ((inportb(card[cardNo].base + MSC) & 0x20)==0x20) {};
}                                                           
//----------------------------------------------------
unsigned char CALLBACK MSTEP3_IS_Y_STOP(unsigned char cardNo)
{
 if (card[cardNo].exist==NO) return NO;
 delay(50);
 // check FIFO1=empty
 if ((inportb(card[cardNo].base + MSC) & 0x01)==0x01) return NO;

 //check /YSTOP
 if ((inportb(card[cardNo].base + MSC) & 0x20)==0x20) return NO;
 return YES;
}
//----------------------------------------------------
void CALLBACK MSTEP3_WAIT_Z(unsigned char cardNo)
{
 if (card[cardNo].exist==NO) return;
 delay(50);
 // check /FIFO1=empty
 while ((inportb(card[cardNo].base + MSC) & 0x01)==0x01) {};

 //wait /ZSTOP
 while ((inportb(card[cardNo].base + MSC) & 0x40)==0x40) {};
 delay(10);
 while ((inportb(card[cardNo].base + MSC) & 0x40)==0x40) {};
}
//----------------------------------------------------
unsigned char CALLBACK MSTEP3_IS_Z_STOP(unsigned char cardNo)
{
 if (card[cardNo].exist==NO) return NO;
 delay(50);
 // check FIFO1=empty
 if ((inportb(card[cardNo].base + MSC) & 0x01)==0x01) return NO;

 //check /ZSTOP
 if ((inportb(card[cardNo].base + MSC) & 0x40)==0x40) return NO;
 return YES;
}
//####################################################
//----------------------------------------------------
void CALLBACK MSTEP3_SET_CONTROL_MODE(unsigned char cardNo,
                              unsigned char x_mode,
                              unsigned char y_mode,
                              unsigned char z_mode)
{                   
  sys.command=sIDLE;
  output_fifo(cardNo, SET_CONTROL_MODE, x_mode, y_mode, z_mode,0,0,0,0,0);
}
//----------------------------------------------------
void CALLBACK MSTEP3_SET_VAR(unsigned char cardNo,
                     unsigned char set_DDA_cycle,
                     unsigned char set_Acc_Dec,
                     unsigned int  set_Low_Speed,
                     unsigned int  set_High_Speed,
                     unsigned int  set_arc_speed)
{                   
  sys.command=sIDLE;
  
  if (set_DDA_cycle==0)  set_DDA_cycle=1;
  if (set_DDA_cycle>254) set_DDA_cycle=254;

  if (set_Acc_Dec==0)  set_Acc_Dec=1;
  if (set_Acc_Dec>250) set_Acc_Dec=250;

  if (set_Low_Speed==0)  set_Low_Speed=1;
  if (set_Low_Speed>500) set_Low_Speed=500;

  if (set_High_Speed < set_Low_Speed) set_High_Speed=set_Low_Speed;
  if (set_High_Speed > 2040)          set_High_Speed=2040;

  output_fifo(cardNo, SET_VAR,set_DDA_cycle,set_Acc_Dec,
                        *(&( *(char *)&set_Low_Speed )+1),
                        *(&( *(char *)&set_Low_Speed )+0),
                        *(&( *(char *)&set_High_Speed )+1),
                        *(&( *(char *)&set_High_Speed )+0),
                        0,0);

  //for interpolation use
  DDA_cycle =set_DDA_cycle;
  Acc_Dec   =set_Acc_Dec;
  Low_Speed =set_Low_Speed;
  High_Speed=set_High_Speed;
  arc_speed =set_arc_speed;
}
//----------------------------------------------------
void CALLBACK MSTEP3_SET_DEFDIR(unsigned char cardNo,
                       unsigned char defdirX,
                       unsigned char defdirY,
                       unsigned char defdirZ)
{ 
  sys.command=sIDLE;
  
  output_fifo(cardNo,SET_DEFDIR,
                ((defdirZ & 0x01) << 2)
              | ((defdirY & 0x01) << 1)
              |  (defdirX & 0x01) ,
              0,0,0,0,0,0,0);
}
//----------------------------------------------------
void CALLBACK MSTEP3_SET_SERVO_ON(unsigned char cardNo,
                          unsigned char sonX,
                          unsigned char sonY,
                          unsigned char sonZ)
{ 
  sys.command=sIDLE;
  
  output_fifo(cardNo, SET_SERVO_ON,
                ((sonZ & 0x01) << 2)
              | ((sonY & 0x01) << 1)
              |  (sonX & 0x01) ,
              0,0,0,0,0,0,0);
}
//----------------------------------------------------
void CALLBACK MSTEP3_SET_ZERO(unsigned char cardNo, unsigned char axis)
{ 
  sys.command=sIDLE;
  
  if ((axis!=X_axis) && (axis!=Y_axis) && (axis!=Z_axis)) return;
  output_fifo(cardNo, SET_ZERO,axis & 0x03,0,0,0,0,0,0,0);
}
//----------------------------------------------------
void CALLBACK MSTEP3_PRESET_POSITION(unsigned char cardNo, unsigned char axis,
                         long preset_position)
{ 
  sys.command=sIDLE;
  
  if ((axis!=X_axis) && (axis!=Y_axis) && (axis!=Z_axis)) return;
  output_fifo(cardNo,
              SET_NONZERO,
              axis & 0x03,
              *(&( *(char *)&preset_position )+3),
              *(&( *(char *)&preset_position )+2),
              *(&( *(char *)&preset_position )+1),
              *(&( *(char *)&preset_position )+0),
              0,0,0);
}
//----------------------------------------------------
void CALLBACK MSTEP3_SET_NC(unsigned char cardNo, unsigned char sw)
{ 
  sys.command=sIDLE;
  
  output_fifo(cardNo, SET_NORMAL_CLOSE, sw,0,0,0,0,0,0,0);
}
//########################################################
void CALLBACK MSTEP3_STOP(unsigned char cardNo, unsigned char axis)
{ 
  sys.command=sIDLE;
  
  if ((axis!=X_axis) && (axis!=Y_axis) && (axis!=Z_axis)) return;
  output_fifo(cardNo, STOP_AXIS, axis & 0x03, 0,0,0,0,0,0,0);
}
//----------------------------------------------------
void CALLBACK MSTEP3_DEC_STOP(unsigned char cardNo, unsigned char axis)
{ 
  sys.command=sIDLE;
  
  if ((axis!=X_axis) && (axis!=Y_axis) && (axis!=Z_axis)) return;
  output_fifo(cardNo, DEC_STOP, axis & 0x03, 0,0,0,0,0,0,0);
}
//----------------------------------------------------
void CALLBACK MSTEP3_RESET_SYSTEM(unsigned char cardNo)
{
unsigned char j;

  sys.command=sIDLE;  
  //--- reset fifo
  outportb(card[cardNo].base + RSTFF1,0x00);
  for (j=0; j<15; j++)
  {
    outportb(card[cardNo].base + WRFF1, 0x00);
  };

  //--- output PCF7
  card[cardNo].op = 0x80;
  outportb(card[cardNo].base + DO, card[cardNo].op);
  delay(200);
  card[cardNo].op = 0x00;
  outportb(card[cardNo].base + DO, card[cardNo].op);
  delay(40);
  WorkFlag=NO;

  for (j=0; j<30; j++)
  {
    outportb(card[cardNo].base + WRFF1, 0x00);
  };

  output_fifo(cardNo,STOP_ALL,0,0,0,0,0,0,0,0);
  output_fifo(cardNo,RESET_SYSTEM,0,0,0,0,0,0,0,0);
}
//----------------------------------------------------
void CALLBACK MSTEP3_STOP_ALL(unsigned char cardNo)
{
unsigned char j;

  sys.command=sIDLE;
  //--- reset fifo
  outportb(card[cardNo].base + RSTFF1,0x00);
  for (j=0; j<15; j++)
  {
    outportb(card[cardNo].base + WRFF1, 0x00);
  };

  //--- output PCF7
  card[cardNo].op |= 0x80;
  outportb(card[cardNo].base + DO, card[cardNo].op);
  delay(200);
  card[cardNo].op &= 0x7F;
  outportb(card[cardNo].base + DO, card[cardNo].op);
  delay(40);
  WorkFlag=NO;      

  for (j=0; j<30; j++)
  {
    outportb(card[cardNo].base + WRFF1, 0x00);
  };
}
//----------------------------------------------------
void CALLBACK MSTEP3_EMG_STOP(unsigned char cardNo)
{
unsigned char j;

  sys.command=sIDLE;  
  //--- reset fifo
  outportb(card[cardNo].base + RSTFF1,0x00);
  for (j=0; j<15; j++)
  {
    outportb(card[cardNo].base + WRFF1, 0x00);
  };
  //--- output PCF7
  card[cardNo].op |= 0x80;
  outportb(card[cardNo].base + DO, card[cardNo].op);
  delay(200);
  card[cardNo].op &= 0x7F;
  outportb(card[cardNo].base + DO, card[cardNo].op);
  delay(40);
  WorkFlag=NO;

  for (j=0; j<30; j++)
  {
    outportb(card[cardNo].base + WRFF1, 0x00);
  };
}      
//----------------------------------------------------------
void CALLBACK MSTEP3_BACK_HOME( unsigned char cardNo,
                        unsigned char axis,
                        unsigned char set_home_speed,
                        unsigned char set_search_speed)
{ 
  sys.command=sIDLE;
  
  output_fifo(cardNo,
              BACK_HOME,
              (axis & 0x03),
              set_home_speed,
              set_search_speed,
              0,0,0,0,0);
}
void CALLBACK MSTEP3_BACK_HOME01(unsigned char cardNo,
                        unsigned char axis,
                        unsigned char set_home_speed)
{ 
  sys.command=sIDLE;
  
  output_fifo(cardNo,
              BACK_HOME01,
              (axis & 0x03),
              set_home_speed,
              0,0,0,0,0,0);
}
//----------------------------------------------------
void CALLBACK MSTEP3_PULSE_MOVE(unsigned char cardNo,
                        unsigned char axis,
                        long pulseN,
                        unsigned int move_speed)
{
unsigned char DIR;
  
  sys.command=sIDLE;
    
  if (move_speed > 2040) move_speed=2040;
  if (move_speed == 0) return;

  if (pulseN>0) DIR=CW; else DIR=CCW;
  pulseN=labs(pulseN);

  output_fifo(cardNo,PULSE_MOVE,((DIR & 0x01) << 4) | (axis & 0x03),
                                  *(&( *(char *)&pulseN )+3),
                                  *(&( *(char *)&pulseN )+2),
                                  *(&( *(char *)&pulseN )+1),
                                  *(&( *(char *)&pulseN )+0),
                                  *(&( *(char *)&move_speed )+1),
                                  *(&( *(char *)&move_speed )+0),
                                  0);
}
//----------------------------------------------------
void CALLBACK MSTEP3_INTP_PULSE(unsigned char cardNo,
                        int Xpulse,
                        int Ypulse,
                        int Zpulse)
{ 
  if (abs(Xpulse) > 2047) return;
  if (abs(Ypulse) > 2047) return;
  if (abs(Zpulse) > 2047) return;

  if (Xpulse < 0) Xpulse = abs(Xpulse) | 0x8000; //CCW
  if (Ypulse < 0) Ypulse = abs(Ypulse) | 0x8000; //CCW
  if (Zpulse < 0) Zpulse = abs(Zpulse) | 0x8000; //CCW

  output_fifo(cardNo,INTP_PULSE,
                *(&( *(char *)&Xpulse )+1),
                *(&( *(char *)&Xpulse )+0),
                *(&( *(char *)&Ypulse )+1),
                *(&( *(char *)&Ypulse )+0),
                *(&( *(char *)&Zpulse )+1),
                *(&( *(char *)&Zpulse )+0),
                0,0 );
}
//--------------------------------------------------------------------------
void CALLBACK MSTEP3_CONSTANT_SPEED(unsigned char cardNo,
                            unsigned char axis,
                            unsigned char dir,
                            unsigned int move_speed)
{ 
  sys.command=sIDLE;
  
  if (move_speed > 2040) move_speed=2040;
  if (move_speed == 0) return;

  output_fifo(cardNo,
              CONSTANT_SPEED,
              ((dir & 0x01) << 4) | (axis & 0x03),
              *(&( *(char *)&move_speed )+1),
              *(&( *(char *)&move_speed )+0),
              0,0,0,0,0);
}
//###########################################################################
//#####################################################################
//#####################################################################
void MSTEP3_INTP_PROFILE(void)
{
unsigned long Dec_Length;
unsigned int  CAD;

    if ( total_pulse < 4*Low_Speed )
    {
       if ( rem_pulse < Low_Speed )
       {
          d_length = (int)rem_pulse;
          profile_state = PROFILE_STOP;
       }
       else
          d_length = Low_Speed;
    }
    else
    {
       Dec_Length = (d_length + Acc_Dec);
       Dec_Length = Dec_Length*Dec_Length/(Acc_Dec*2);

       if ((Dec_Flag == 0) && (rem_pulse > Dec_Length))
       {
          if (d_length < High_Speed)  d_length += Acc_Dec;
          if (d_length >= High_Speed) d_length = High_Speed;
       }
       else
       {
          Dec_Flag=1;
          CAD =(unsigned int)(((unsigned long)d_length*
               (unsigned long)d_length )/(2*rem_pulse));

          if ((int)(d_length - CAD) < (int)Acc_Dec )
             d_length = Acc_Dec;
          else
             d_length -= CAD;

          if (rem_pulse <= d_length)
          {
             d_length = (unsigned int)rem_pulse;
             profile_state = PROFILE_STOP;
          }
       }
    }
    rem_pulse -= d_length;
}
//----------------------------------------------------
void MSTEP3_INTP_PROFILE_CSPD(unsigned int spd)
{
    if (rem_pulse > spd)
    {
       d_length = spd;
    }
    else
    {
       d_length = (unsigned int)rem_pulse;
       profile_state = PROFILE_STOP;
    }

    rem_pulse -= d_length;
}

//----------------------------------------------------------
unsigned char CALLBACK MSTEP3_XYZ_CORE(unsigned char cardNo,
                       long x,
                       long y,
                       long z,
                       unsigned int speed,
                       unsigned char acc_mode)
{
  switch (state)
  {
    case PHASE1:
       WorkFlag=YES;
       if ((x==0) && (y==0) && (z==0)) return READY;

       if (speed>2040) speed=2040;
       temp_speed = High_Speed;

       if (x<0) x_dir=-1; else x_dir=1;
       if (y<0) y_dir=-1; else y_dir=1;
       if (z<0) z_dir=-1; else z_dir=1;

       total_pulseX=labs(x);
       total_pulseY=labs(y);
       total_pulseZ=labs(z);

       MaxAxis=1;
       //if ((total_pulseX>total_pulseY) && (total_pulseX>total_pulseZ))
       //   MaxAxis=1;
       if ((total_pulseY>total_pulseX) && (total_pulseY>total_pulseZ))
          MaxAxis=2;
       if ((total_pulseZ>total_pulseX) && (total_pulseZ>total_pulseY))
	  MaxAxis=3;

       switch (MaxAxis)
       {
        case 1: total_pulse=total_pulseX;
                break;
        case 2: total_pulse=total_pulseY;
                break;
        case 3: total_pulse=total_pulseZ;
                break;
       }

       total_length=sqrt(   (double)(total_pulseX)*(double)(total_pulseX)
                          + (double)(total_pulseY)*(double)(total_pulseY)
                          + (double)(total_pulseZ)*(double)(total_pulseZ)
                        );

       speed = (unsigned int)(speed*(double)(total_pulse)/total_length);
       High_Speed = speed;

       rem_pulse=total_pulse;
       Dec_Flag=0;
       d_length=0;
       pulse_R1=0;
       pulse_R2=0;
       profile_state = PROFILE_BUSY;
       state=PHASE2;
       return BUSY;
       //break;

    case PHASE2:
       do
       {
          //---get d_length-----------------------------
          if (acc_mode==0)
             MSTEP3_INTP_PROFILE();
          else
             MSTEP3_INTP_PROFILE_CSPD(speed);

          switch (MaxAxis)
	  {
            case 1:
                    dx=d_length;
                    dy=(unsigned int) ((total_pulseY*dx+pulse_R1)/total_pulseX);
                    pulse_R1 =         (total_pulseY*dx+pulse_R1)%total_pulseX;
                    dz=(unsigned int) ((total_pulseZ*dx+pulse_R2)/total_pulseX);
                    pulse_R2 =         (total_pulseZ*dx+pulse_R2)%total_pulseX;
                    break;
            case 2:
                    dy=d_length;
                    dx=(unsigned int) ((total_pulseX*dy+pulse_R1)/total_pulseY);
                    pulse_R1 =         (total_pulseX*dy+pulse_R1)%total_pulseY;
                    dz=(unsigned int) ((total_pulseZ*dy+pulse_R2)/total_pulseY);
                    pulse_R2 =         (total_pulseZ*dy+pulse_R2)%total_pulseY;
                    break;
	    case 3:
                    dz=d_length;
                    dx=(unsigned int) ((total_pulseX*dz+pulse_R1)/total_pulseZ);
		    pulse_R1 =         (total_pulseX*dz+pulse_R1)%total_pulseZ;
		    dy=(unsigned int) ((total_pulseY*dz+pulse_R2)/total_pulseZ);
		    pulse_R2 =         (total_pulseY*dz+pulse_R2)%total_pulseZ;
		    break;
	  }
	  MSTEP3_INTP_PULSE(cardNo, x_dir*dx, y_dir*dy, z_dir*dz);

       } while ( (profile_state==PROFILE_BUSY) && (fifo_full==NO) );

       if (profile_state==PROFILE_STOP)
       {
	  High_Speed = temp_speed;
          state=PHASE1;
          return READY;
       }
       else
       {
          state=PHASE2;
          return BUSY;
       }
       //break;
  };
  return READY;
}

void CALLBACK MSTEP3_INTP_XYZ(unsigned char cardNo,
		      long x,
                      long y,
                      long z,
                      unsigned int speed)
{
  state=PHASE1;
  sys.command=sIDLE;  
  while(MSTEP3_XYZ_CORE(cardNo, x, y, z, speed, 0)!=READY);
}

//----------------------------------------------------------
unsigned char CALLBACK MSTEP3_LINE_CORE(unsigned char cardNo,
                                 unsigned char plane,
                                 long x,long y,
                                 unsigned int speed,
                                 unsigned char acc_mode)
{
  switch (state)
  {
    case PHASE1:
       WorkFlag=YES;
       if ((x==0) && (y==0)) return READY;

       if (speed>2040) speed=2040;
       temp_speed = High_Speed;
       High_Speed = speed;

       if (x<0) x_dir=-1; else x_dir=1;
       if (y<0) y_dir=-1; else y_dir=1;
       total_pulseX=labs(x);
       total_pulseY=labs(y);

       if (total_pulseX>total_pulseY)
           total_pulse=total_pulseX;
       else
	   total_pulse=total_pulseY;

       total_length=sqrt(   (double)(total_pulseX)*(double)(total_pulseX)
                          + (double)(total_pulseY)*(double)(total_pulseY)
                        );

       speed = (unsigned int)(speed*(double)(total_pulse)/total_length);
       High_Speed = speed;

       rem_pulse=total_pulse;
       Dec_Flag=0;
       d_length=0;
       pulse_R=0;
       profile_state = PROFILE_BUSY;
       state=PHASE2;
       return BUSY;
       //break;

   case PHASE2:
       do
       {
          if (acc_mode==0)
              MSTEP3_INTP_PROFILE();
          else
              MSTEP3_INTP_PROFILE_CSPD(speed);

          if (total_pulseX>total_pulseY)
          {
             dx=d_length;
             dy=(unsigned int) ((total_pulseY*dx+pulse_R)/total_pulseX);
             pulse_R =(total_pulseY*dx+pulse_R)%total_pulseX;
          }
          else
          {
             dy=d_length;
             dx=(unsigned int) ((total_pulseX*dy+pulse_R)/total_pulseY);
	     pulse_R =(total_pulseX*dy+pulse_R)%total_pulseY;
          }

          switch (plane)
	  {
            case XY_plane:
                  MSTEP3_INTP_PULSE(cardNo, x_dir*dx, y_dir*dy, 0);
                  break;
            case XZ_plane:
                  MSTEP3_INTP_PULSE(cardNo, x_dir*dx, 0, y_dir*dy);
                  break;
            case YZ_plane:
                  MSTEP3_INTP_PULSE(cardNo, 0, x_dir*dx, y_dir*dy);
                  break;
          };
       } while ((profile_state==PROFILE_BUSY) && (fifo_full==NO) );

       if (profile_state==PROFILE_STOP)
       {
	  High_Speed = temp_speed;
	  state=PHASE1;
	  return READY;
       }
       else
       {
	  state=PHASE2;
	  return BUSY;
       }
       //break;
  };
  return READY;
}
//----------------------------------------------------------
void CALLBACK MSTEP3_INTP_LINE01(unsigned char cardNo,
                         unsigned char plane,
                         long x,
                         long y,
                         unsigned int speed)
{
  sys.command=sIDLE;
  
    WorkFlag=YES;
    if ((x==0) && (y==0)) return;

    if (speed>2040) speed=2040;
    temp_speed = High_Speed;
    High_Speed = speed;

    if (x<0) x_dir=-1; else x_dir=1;
    if (y<0) y_dir=-1; else y_dir=1;
    total_pulseX=labs(x);
    total_pulseY=labs(y);

    if (total_pulseX>total_pulseY)
        total_pulse=total_pulseX;
    else
        total_pulse=total_pulseY;

    total_length=sqrt(   (double)(total_pulseX)*(double)(total_pulseX)
                       + (double)(total_pulseY)*(double)(total_pulseY)
                     );

    speed = (unsigned int)(speed*(double)(total_pulse)/total_length);
    High_Speed = speed;

    rem_pulse=total_pulse;
    Dec_Flag=0;
    d_length=0;
    pulse_R=0;
    profile_state = PROFILE_BUSY;
    do
    {
           MSTEP3_INTP_PROFILE();

           if (total_pulseX>total_pulseY)
            {
                    dx=d_length;
                    dy=(unsigned int) ((total_pulseY*dx+pulse_R)/total_pulseX);
                    pulse_R =(total_pulseY*dx+pulse_R)%total_pulseX;
                  }
           else
                  {
                    dy=d_length;
                    dx=(unsigned int) ((total_pulseX*dy+pulse_R)/total_pulseY);
                    pulse_R =(total_pulseX*dy+pulse_R)%total_pulseY;
                  }

           switch (plane)
	   {
            case XY_plane:
                     MSTEP3_INTP_PULSE(cardNo, x_dir*dx, y_dir*dy, 0);
                     break;
            case XZ_plane:
                     MSTEP3_INTP_PULSE(cardNo, x_dir*dx, 0, y_dir*dy);
                     break;
            case YZ_plane:
                     MSTEP3_INTP_PULSE(cardNo, 0, x_dir*dx, y_dir*dy);
                     break;
           };

    } while (profile_state==PROFILE_BUSY);

    High_Speed = temp_speed;
}
void MSTEP3_INTP_LINE(unsigned char cardNo,
                       long x,
                       long y,
                       unsigned int speed)
{
  MSTEP3_INTP_LINE01(cardNo, XY_plane, x, y, speed);
}

//-----------------------------------------------------------
unsigned char CALLBACK MSTEP3_CIRCLE_CORE(unsigned char cardNo,
                           unsigned char plane,
                           long x,
                           long y,
                           unsigned char dir,
                           unsigned int speed,
                           unsigned char acc_mode)
{
   switch (state)
   {
      case PHASE1:
         WorkFlag=YES;
         if ((x==0) && (y==0)) return READY;
	 if (speed>2040) speed=2040;
         temp_speed = High_Speed;
         High_Speed = speed;

         radius = sqrt((double)(x)*(double)(x)+(double)(y)*(double)(y));
         total_pulse = (unsigned long) (2.0*pi*radius);
         Xr=(double)(-x);
         Yr=(double)(-y);

         if (Xr!=0.0)
	 {
            theta = atan(Yr/Xr);
            if (Xr<0.0) theta += pi;
         }
         else
            if (Yr>0.0) theta=pi/2.0; else theta=-pi/2.0;

         Xn=0.0; Yn=0.0;
         rem_pulse=total_pulse;
         Dec_Flag=0;
	 d_length=0;
         profile_state = PROFILE_BUSY;
         state=PHASE2;
         return BUSY;
	 //break;

      case PHASE2:
         do
         {
             //---get d_length-----------------------------
	     if (acc_mode==0)
                MSTEP3_INTP_PROFILE();
             else
                MSTEP3_INTP_PROFILE_CSPD(speed);

             d_theta = (double)(d_length) / radius;
             if (dir == CW)
             {
                 xx1 = x + radius*cos(theta - d_theta);
                 yy1 = y + radius*sin(theta - d_theta);
		 theta -= d_theta;
             }
             else
             {
		 xx1 = x + radius*cos(theta + d_theta);
                 yy1 = y + radius*sin(theta + d_theta);
                 theta += d_theta;
             }
             if (profile_state == PROFILE_STOP)
             {
		 xx1=0;
                 yy1=0;
             }
             dx = (int)(xx1-Xn);
             dy = (int)(yy1-Yn);
             Xn += (double)(dx);
             Yn += (double)(dy);

             switch (plane)
             {
	      case XY_plane:
                       MSTEP3_INTP_PULSE(cardNo,dx,dy,0);
                       break;
              case XZ_plane:
		       MSTEP3_INTP_PULSE(cardNo,dx,0,dy);
                       break;
              case YZ_plane:
                       MSTEP3_INTP_PULSE(cardNo,0,dx,dy);
                       break;
             };
	 } while ((profile_state==PROFILE_BUSY) && (fifo_full==NO) );

	 if (profile_state==PROFILE_STOP)
	 {
	    High_Speed = temp_speed;
	    state=PHASE1;
	    return READY;
	 }
	 else
	 {
	    state=PHASE2;
            return BUSY;
         }
	 //break;
   };
   return READY;
}
//-----------------------------------------------------------
void CALLBACK MSTEP3_INTP_CIRCLE01(unsigned char cardNo,
                           unsigned char plane,
                           long x,
                           long y,
                           unsigned char dir,
                           unsigned int speed)
{ 
  sys.command=sIDLE;
  
    dx=0;
    dy=0;

    WorkFlag=YES;
    if ((x==0) && (y==0)) return;

    if (speed>2040) speed=2040;
    temp_speed = High_Speed;
    High_Speed = speed;

    radius = sqrt((double)(x)*(double)(x)+(double)(y)*(double)(y));
    total_pulse = (unsigned long) (2.0*pi*radius);
    Xr=(double)(-x);
    Yr=(double)(-y);

    if (Xr!=0.0)
    {
       theta = atan(Yr/Xr);
       if (Xr<0.0) theta += pi;
    }
    else
       if (Yr>0.0) theta=pi/2.0; else theta=-pi/2.0;

    Xn=0.0; Yn=0.0;
    rem_pulse=total_pulse;
    Dec_Flag=0;
    d_length=0;
    profile_state = PROFILE_BUSY;

    do
    {
        MSTEP3_INTP_PROFILE();
        d_theta = (double)(d_length) / radius;
        if (dir == CW)
        {
            xx1 = x + radius*cos(theta - d_theta);
            yy1 = y + radius*sin(theta - d_theta);
            theta -= d_theta;
        }
        else
        {
            xx1 = x + radius*cos(theta + d_theta);
            yy1 = y + radius*sin(theta + d_theta);
            theta += d_theta;
        }
        if (profile_state == PROFILE_STOP)
        {
            xx1=0;
            yy1=0;
        }
        dx = (int)(xx1-Xn);
        dy = (int)(yy1-Yn);
        Xn += (double)(dx);
        Yn += (double)(dy);

        switch (plane)
        {
         case XY_plane:
                  MSTEP3_INTP_PULSE(cardNo,dx,dy,0);
                  break;
         case XZ_plane:
                  MSTEP3_INTP_PULSE(cardNo,dx,0,dy);
                  break;
         case YZ_plane:
                  MSTEP3_INTP_PULSE(cardNo,0,dx,dy);
                  break;
        };

    } while (profile_state==PROFILE_BUSY) ;

    High_Speed = temp_speed;
}
void MSTEP3_INTP_CIRCLE(unsigned char cardNo,
                         long x,
                         long y,
			 unsigned char dir,
                         unsigned int speed)
{
   MSTEP3_INTP_CIRCLE01(cardNo, XY_plane ,x,y,dir,speed);
}


//--------------------------------------------------------------------------------
unsigned char CALLBACK MSTEP3_ARC_CORE(unsigned char cardNo,
                        unsigned char plane,
                        long x,
                        long y,
                        long R,
                        unsigned char dir,
                        unsigned int speed,
                        unsigned char acc_mode)
{
   switch (state)
   {
      case PHASE1:
         WorkFlag=YES;
         if (R==0) return READY;
         if (speed>2040) speed=2040;

         temp_speed = High_Speed;
         High_Speed = speed;

         radius = (double)(labs(R));
         xx1     = (double)(x);
         yy1     = (double)(y);

         if (radius < sqrt(xx1*xx1+yy1*yy1)/2.0) return READY;

         th = 2.0*asin(sqrt(xx1*xx1+yy1*yy1)/2.0/radius);

         // calculate th1
	 if ((R > 0) && (dir == CW)) th1 = (pi-th)/2.0;
         if ((R > 0) && (dir == CCW)) th1 = -(pi-th)/2.0;
	 if ((R < 0) && (dir == CCW)) th1 = (pi-th)/2.0;
         if ((R < 0) && (dir == CW)) th1 = -(pi-th)/2.0;

         // calculate th2 = atan(yy1/xx1)
         if (xx1!=0.0)
         {
            th2 = atan(yy1/xx1);
            if (xx1<0.0) th2 += pi;
         }
         else
            if (yy1>0.0) th2=pi/2.0; else th2=-pi/2.0;

         // calculate center of arc
         xc = xx1/2.0 + radius*sin(th1)*sin(th2);
         yc = yy1/2.0 - radius*sin(th1)*cos(th2);

         // calculate theta = atan(-yc/-xc)
         if (-xc!=0.0)
	 {
            theta = atan(-yc/-xc);
	    if (-xc<0.0) theta += pi;
         }
         else
            if (-yc>0.0) theta=pi/2.0; else theta=-pi/2.0;

         // calculate total pulse
         if (R > 0)
            total_pulse = (unsigned long) (th*radius);
         else
            total_pulse = (unsigned long) ((2.0*pi-th)*radius);

         x_old=0.0; y_old=0.0;
         rem_pulse=total_pulse;
         Dec_Flag=0;
         d_length=0;
         profile_state = PROFILE_BUSY;
         state=PHASE2;
         return BUSY;
	 //break;

      case PHASE2:
         do
         {
            //---get d_length-----------------------------
            if (acc_mode==0)
               MSTEP3_INTP_PROFILE();
            else
               MSTEP3_INTP_PROFILE_CSPD(speed);

            d_theta = (double)(d_length) / radius;
            if (dir == CW)
            {
                x_new = xc + radius*cos(theta - d_theta);
                y_new = yc + radius*sin(theta - d_theta);
                theta -= d_theta;
            }
            else
            {
		x_new = xc + radius*cos(theta + d_theta);
                y_new = yc + radius*sin(theta + d_theta);
                theta += d_theta;
            }
            if (profile_state == PROFILE_STOP)
            {
                x_new=xx1;
                y_new=yy1;
            }
            dx = (int)(x_new-x_old);
            dy = (int)(y_new-y_old);
            x_old += (double)(dx);
            y_old += (double)(dy);

            switch (plane)
            {
             case XY_plane:
                      MSTEP3_INTP_PULSE(cardNo,dx,dy,0);
                      break;
             case XZ_plane:
		      MSTEP3_INTP_PULSE(cardNo,dx,0,dy);
                      break;
             case YZ_plane:
                      MSTEP3_INTP_PULSE(cardNo,0,dx,dy);
                      break;
            };
	 } while ((profile_state==PROFILE_BUSY) && (fifo_full==NO) );

         if (profile_state==PROFILE_STOP)
         {
            High_Speed = temp_speed;
            state=PHASE1;
            return READY;
         }
         else
         {
            state=PHASE2;
            return BUSY;
         }
	 //break;
   };
   return READY;
}
//--------------------------------------------------------------------------------
void CALLBACK MSTEP3_INTP_ARC01(unsigned char cardNo,
                        unsigned char plane,
                        long x,
                        long y,
                        long R,
                        unsigned char dir,
                        unsigned int speed)
{
  sys.command=sIDLE;
  
    dx=0;
    dy=0;

    WorkFlag=YES;
    if (R==0) return;
    if (speed>2040) speed=2040;

    temp_speed = High_Speed;
    High_Speed = speed;

    radius = (double)(labs(R));
    xx1     = (double)(x);
    yy1     = (double)(y);

    if (radius < sqrt(xx1*xx1+yy1*yy1)/2.0) return;

    th = 2.0*asin(sqrt(xx1*xx1+yy1*yy1)/2.0/radius);

    // calculate th1
    if ((R > 0) && (dir == CW)) th1 = (pi-th)/2.0;
    if ((R > 0) && (dir == CCW)) th1 = -(pi-th)/2.0;
    if ((R < 0) && (dir == CCW)) th1 = (pi-th)/2.0;
    if ((R < 0) && (dir == CW)) th1 = -(pi-th)/2.0;

    // calculate th2 = atan(yy1/xx1)
    if (xx1!=0.0)
    {
       th2 = atan(yy1/xx1);
       if (xx1<0.0) th2 += pi;
    }
    else
       if (yy1>0.0) th2=pi/2.0; else th2=-pi/2.0;

    // calculate center of arc
    xc = xx1/2.0 + radius*sin(th1)*sin(th2);
    yc = yy1/2.0 - radius*sin(th1)*cos(th2);

    // calculate theta = atan(-yc/-xc)
    if (-xc!=0.0)
    {
       theta = atan(-yc/-xc);
       if (-xc<0.0) theta += pi;
    }
    else
       if (-yc>0.0) theta=pi/2.0; else theta=-pi/2.0;

    // calculate total pulse
    if (R > 0)
       total_pulse = (unsigned long) (th*radius);
    else
       total_pulse = (unsigned long) ((2.0*pi-th)*radius);

    x_old=0.0; y_old=0.0;
    rem_pulse=total_pulse;
    Dec_Flag=0;
    d_length=0;
    profile_state = PROFILE_BUSY;

    do
    {
        MSTEP3_INTP_PROFILE();
        d_theta = (double)(d_length) / radius;
        if (dir == CW)
        {
            x_new = xc + radius*cos(theta - d_theta);
            y_new = yc + radius*sin(theta - d_theta);
            theta -= d_theta;
        }
        else
        {
            x_new = xc + radius*cos(theta + d_theta);
            y_new = yc + radius*sin(theta + d_theta);
            theta += d_theta;
        }
        if (profile_state == PROFILE_STOP)
        {
            x_new=xx1;
            y_new=yy1;
        }
        dx = (int)(x_new-x_old);
        dy = (int)(y_new-y_old);
        x_old += (double)(dx);
        y_old += (double)(dy);

        switch (plane)
        {
         case XY_plane:
                  MSTEP3_INTP_PULSE(cardNo,dx,dy,0);
                  break;
         case XZ_plane:
                  MSTEP3_INTP_PULSE(cardNo,dx,0,dy);
                  break;
         case YZ_plane:
                  MSTEP3_INTP_PULSE(cardNo,0,dx,dy);
                  break;
        };
    } while (profile_state==PROFILE_BUSY);

    High_Speed = temp_speed;
}

void CALLBACK MSTEP3_INTP_ARC(unsigned char cardNo,
                      long x,
                      long y,
                      long R,
                      unsigned char dir,
                      unsigned int speed)
{
  MSTEP3_INTP_ARC01(cardNo, XY_plane ,x,y,R,dir,speed);
}

//#####################################################################
void intp_state_machine()
{
unsigned char status;

  switch (sys.command)
  {
     case sXYZ:
        status= MSTEP3_XYZ_CORE(sys.cardNo, sys.x, sys.y, sys.z,
                                  sys.speed, sys.acc_mode);
        if (status==READY)
        {
           state=PHASE1;
           sys.command=sIDLE;
        };   
        break;

     case sLINE: 
        status= MSTEP3_LINE_CORE(sys.cardNo, sys.plane, sys.x, sys.y, 
                                   sys.speed, sys.acc_mode);
        if (status==READY)
        {
           state=PHASE1;
           sys.command=sIDLE;
        };                                   
        break;
     case sCIRCLE: 
        status= MSTEP3_CIRCLE_CORE(sys.cardNo, sys.plane, sys.x, sys.y,
                                   sys.dir, sys.speed, sys.acc_mode);
        if (status==READY)
        {
           state=PHASE1;
           sys.command=sIDLE;
        };     
        break;
     case sARC: 
        status= MSTEP3_ARC_CORE(sys.cardNo, sys.plane, sys.x, sys.y,
                                sys.R, sys.dir, sys.speed, sys.acc_mode);
        if (status==READY)
        {
           state=PHASE1;
           sys.command=sIDLE;
        };     
        break;

     case sIDLE:
     default: 
        state=PHASE1;
        break;
  }
}
//-------------------------------------------------------------
void CALLBACK MSTEP3_INTP_XYZ02(unsigned char cardNo,
                       long x,
                       long y,
                       long z,
                       unsigned int speed,
                       unsigned char acc_mode)
{           
   sys.command=sXYZ;
   sys.cardNo=cardNo;
   sys.x=x;
   sys.y=y;
   sys.z=z;
   sys.speed=speed;
   sys.acc_mode=acc_mode;

   state=PHASE1;
   intp_state_machine();
}                                      
void CALLBACK MSTEP3_INTP_LINE02(unsigned char cardNo,
                                 unsigned char plane,
                                 long x,long y,
                                 unsigned int speed,
                                 unsigned char acc_mode)
{
   sys.command=sLINE;
   sys.cardNo=cardNo;
   sys.plane=plane;
   sys.x=x;
   sys.y=y;
   sys.speed=speed;
   sys.acc_mode=acc_mode;

   state=PHASE1;
   intp_state_machine();
}                                        

void CALLBACK MSTEP3_INTP_CIRCLE02(unsigned char cardNo,
                           unsigned char plane,
                           long x,
                           long y,
                           unsigned char dir,
                           unsigned int speed,
                           unsigned char acc_mode)
{
   sys.command=sCIRCLE;
   sys.cardNo=cardNo;
   sys.plane=plane;
   sys.x=x;
   sys.y=y;     
   sys.dir=dir;
   sys.speed=speed;
   sys.acc_mode=acc_mode;

   state=PHASE1;
   intp_state_machine();
}                                   
void CALLBACK MSTEP3_INTP_ARC02(unsigned char cardNo,
                        unsigned char plane,
                        long x,
                        long y,
                        long R,
                        unsigned char dir,
                        unsigned int speed,
                        unsigned char acc_mode)
{         
   sys.command=sARC;
   sys.cardNo=cardNo;
   sys.plane=plane;
   sys.x=x;
   sys.y=y;        
   sys.R=R;
   sys.dir=dir;
   sys.speed=speed;
   sys.acc_mode=acc_mode;

   state=PHASE1;
   intp_state_machine();
}                                
unsigned char CALLBACK MSTEP3_INTP_STOP()
{
#ifdef DOS
  //------------------------------------
  // execute interpolation command
  intp_state_machine();
#endif

  if (sys.command==sIDLE) return READY;
  else                    return BUSY;
}
//#####################################################################
void CALLBACK MSTEP3_GET_CARD(unsigned char cardNo)
{
unsigned char latch_counter=0;
unsigned char ff2_empt=0x00;
unsigned char temp1,temp2,temp3,temp4;
unsigned char ABX,j,k;

  if (card[cardNo].exist==NO) return;

#ifndef DOS        
  //------------------------------------
  // execute interpolation command
  intp_state_machine();
#endif   

  //------------------------------------
  do
  {
         // to get sync signal
         ff2_empt = 0x04 & inportb( card[cardNo].base + MSC);
         if (ff2_empt==0x00) return;   //---------------------->  return

         card[cardNo].sync2 = inportb( card[cardNo].base + RDFF2);

         if (   (card[cardNo].sync1==sync_pattern1)
                  && (card[cardNo].sync2==sync_pattern2) )
         {
                 // command sync OK, then to get command from FIFO
                 latch_counter = 0;
                 do
                 {
                        ff2_empt = 0x04 & inportb( card[cardNo].base + MSC);
                        if (ff2_empt==0x04)  //
                        {
                          pc_byte[latch_counter] = inportb( card[cardNo].base + RDFF2);
                          latch_counter++;
                          //--------- receive data ---------------------------------------
                          if (latch_counter > 31)
                          {
                                 ABX =0;
                                 for (j=0; j<31; j++)
                                         ABX = ABX ^ pc_byte[j];

                                 if (ABX != pc_byte[31]) break;  //---------> protect

                                 //change MSB ->LSB
                                 for (k=7; k<=27; k =k+4)
                                 {
                                        temp1=pc_byte[k]; temp2=pc_byte[k+1];
                                        temp3=pc_byte[k+2]; temp4=pc_byte[k+3];
                                        pc_byte[k]=temp4; pc_byte[k+1]=temp3;
                                        pc_byte[k+2]=temp2; pc_byte[k+3]=temp1;
                                 };

                                 //-------------
                                 card[cardNo].sys        =pc_byte[0];
                                 card[cardNo].limit      =pc_byte[1];
                                 card[cardNo].p1         =pc_byte[2];
                                 card[cardNo].InWorkNo   =pc_byte[3];
                                 card[cardNo].X_state    =pc_byte[4];
                                 card[cardNo].Y_state    =pc_byte[5];
                                 card[cardNo].Z_state    =pc_byte[6];
                                 card[cardNo].command_X  =(*(long *)(pc_byte +7));
                                 card[cardNo].position_X =(*(long *)(pc_byte +11));
                                 card[cardNo].command_Y  =(*(long *)(pc_byte +15));
                                 card[cardNo].position_Y =(*(long *)(pc_byte +19));
                                 card[cardNo].command_Z  =(*(long *)(pc_byte +23));
                                 card[cardNo].position_Z =(*(long *)(pc_byte +27));
                          }
                        }
                 } while ( latch_counter<32 );

                 latch_counter = 0;
         }
         // shift sync2 -> sync1
         card[cardNo].sync1 = card[cardNo].sync2;
  } while (1);
}
//###########################################################
unsigned char CALLBACK MSTEP3_GET_SYS(unsigned char cardNo)
{
  return card[cardNo].sys;
}
unsigned char CALLBACK MSTEP3_GET_LIMIT(unsigned char cardNo)
{
  return card[cardNo].limit;
}
unsigned char CALLBACK MSTEP3_GET_P1(unsigned char cardNo)
{
  return card[cardNo].p1;
}
long CALLBACK MSTEP3_GET_XC(unsigned char cardNo)
{
  return card[cardNo].command_X;
}
long CALLBACK MSTEP3_GET_XP(unsigned char cardNo)
{
  return card[cardNo].position_X;
}
long CALLBACK MSTEP3_GET_YC(unsigned char cardNo)
{
  return card[cardNo].command_Y;
}
long CALLBACK MSTEP3_GET_YP(unsigned char cardNo)
{
  return card[cardNo].position_Y;
}
long CALLBACK MSTEP3_GET_ZC(unsigned char cardNo)
{
  return card[cardNo].command_Z;
}
long CALLBACK MSTEP3_GET_ZP(unsigned char cardNo)
{
  return card[cardNo].position_Z;
}
//###########################################################




