#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <string.h>

#ifdef target_mstep3
#include "msw-mstep3.h"
#define MAX_CARDNO 16
#endif

#ifdef target_enc600
#include "msw-enc600.h"
#define MAX_CARDNO 20
#endif

#define LINE_SIZE 128
#define NAME_SIZE 16

typedef struct
{
	int fd;
	int base;
	int iorange;

}fdArray;
fdArray fds[MAX_CARDNO];	//0-MAX_CARDNO

#ifdef target_mstep3
int MSTEP3_INIT(unsigned char cardNo, unsigned int address)
{
	FILE *fd_proc;
	char dev_file[NAME_SIZE + 1];
	char rbuf[LINE_SIZE + 1];
	int fd_dev;
	ixpio_devinfo_t dev;
	ixpio_reg_t reg;
	int cardNum = 0;

	if (cardNo > MAX_CARDNO)
		return FAILURE;

	fd_proc = fopen(IXPIO_PROC_FILE, "r");
	if (!fd_proc) {
		puts("There is not PIO device found.");
		return FAILURE;
	}

	while (fgets(rbuf, LINE_SIZE, fd_proc)) {
		if (!strncmp(rbuf, "dev:", 4)) {
			// make up available device file name 
			sprintf(dev_file, "/dev/%s", strtok(rbuf + 4, " "));
			
			// open device
			fd_dev = open(dev_file, O_RDWR);
			if (fd_dev < 0) {
				printf("Cannot open device file \"%s.\"\n", dev_file);
				close(fd_dev);
				fclose(fd_proc);
				return FAILURE;
			}

			// get device information 
			if (ioctl(fd_dev, IXPIO_GET_INFO, &dev)) {
				puts("Failure of getting device information.\n");
				close(fd_dev);
				fclose(fd_proc);
				return FAILURE;
			}
			if (dev.csid == PISO_PS300) {
				if (dev.hid == address){
					fds[cardNo].fd = fd_dev;
					fds[cardNo].base = dev.base;
					fds[cardNo].iorange = 0x100;
					card[cardNo].base = dev.base;
					card[cardNo].op = 0;
					card[cardNo].workNo = 1;
					card[cardNo].sync1 = 0;
					card[cardNo].sync2 = 0;
					
					// set reset = 0
					reg.id = IXPIO_TJ_CNTL;
					reg.value = 0x00;
					if (ioctl(fd_dev, IXPIO_WRITE_REG, &reg))
						puts("Failure of set reset = 0\n");
	
					// set reset = 1 
					reg.value = 0x01;
					if (ioctl(fd_dev, IXPIO_WRITE_REG, &reg))
						puts("Failure of set reset = 1\n");
			
					// set all AUX are input 
					reg.id = IXPIO_TJ_AUXC;
					reg.value = 0x00;
					if (ioctl(fd_dev, IXPIO_WRITE_REG, &reg))
						puts("Failure of IXPIO_WRITE_REG:TJ_AUXC\n");
					++cardNum;
					card[cardNo].exist = MSTEP3_EXIST(cardNo);
					break;
				}
			}else{ 
				close(fd_dev);
			}
		}
	}

	fclose(fd_proc);
	if (cardNum ==0) return FAILURE;
	WorkFlag = NO;
	state = PHASE1;
	sys.command = sIDLE;
	return card[cardNo].exist;
}
#elif target_enc600
int ENC_INIT(unsigned char cardNo, unsigned int address)
{
	FILE *fd_proc;
	char dev_file[NAME_SIZE + 1];
	char rbuf[LINE_SIZE + 1];
	int fd_dev;
	ixpio_devinfo_t dev;
	ixpio_reg_t reg;
	int cardNum = 0;

	if (cardNo > MAX_CARDNO)
		return FAILURE;

	fd_proc = fopen(IXPIO_PROC_FILE, "r");
	if (!fd_proc) {
		puts("There is not PIO device found.\n");
		return FAILURE;
	}

	while (fgets(rbuf, LINE_SIZE, fd_proc)) {
		if (!strncmp(rbuf, "dev:", 4)) {
			// make up available device file name 
			sprintf(dev_file, "/dev/%s", strtok(rbuf + 4, " "));
			
			// open device
			fd_dev = open(dev_file, O_RDWR);
			if (fd_dev < 0) {
				printf("Cannot open device file \"%s.\"\n", dev_file);
				close(fd_dev);
				fclose(fd_proc);
				return FAILURE;
			}

			// get device information 
			if (ioctl(fd_dev, IXPIO_GET_INFO, &dev)) {
				puts("Failure of getting device information.\n");
				close(fd_dev);
				fclose(fd_proc);
				return FAILURE;
			}
			if (dev.csid == PISO_ENC600 || dev.csid == PISO_ENC300) {
				if (dev.hid == address){
					fds[cardNo].fd = fd_dev;
					fds[cardNo].base = dev.base;
					fds[cardNo].iorange = 0xff;
					card[cardNo].base = dev.base;
					card[cardNo].op = 0;
	
					// set reset = 0
					reg.id = IXPIO_TJ_CNTL;
					reg.value = 0x00;
					if (ioctl(fd_dev, IXPIO_WRITE_REG, &reg))
						puts("Failure of set reset = 0\n");
	
					// set reset = 1 
					reg.value = 0x01;
					if (ioctl(fd_dev, IXPIO_WRITE_REG, &reg))
						puts("Failure of set reset = 1\n");
			
					// set all AUX are input 
					reg.id = IXPIO_TJ_AUXC;
					reg.value = 0x00;
					if (ioctl(fd_dev, IXPIO_WRITE_REG, &reg))
						puts("Failure of IXPIO_WRITE_REG:TJ_AUXC\n");
					cardNum++;
					break;
				}
			}else{ 
				close(fd_dev);
			}
		}
	}

	fclose(fd_proc);
	if (cardNum ==0) return FAILURE;
	else return SUCCESS;
}
#endif
unsigned int myinb(unsigned int cdata)
{
	ixpio_reg_t reg;
	int x;
	x=0;
	for (x = 0; x < MAX_CARDNO; x++)
	{
		if ((cdata >= fds[x].base) && (cdata < (fds[x].base + fds[x].iorange)))
		{
			switch (cdata - fds[x].base) {
#ifdef target_mstep3
				case FIFO2:
					reg.id = IXPIO_FIFO2;
					if (ioctl(fds[x].fd, IXPIO_READ_REG, & reg))
						puts("Failure of IXPIO_READ_REG:FIFO2\n");
					break;

				case MR:
					reg.id = IXPIO_MR;
					if (ioctl(fds[x].fd, IXPIO_READ_REG, & reg))
						puts("Failure of IXPIO_READ_REG:MR\n");
					break;
		
				case DI:
					reg.id = IXPIO_DI;
					if (ioctl(fds[x].fd, IXPIO_READ_REG, & reg))
						puts("Failure of IXPIO_READ_REG:DI\n");
					break;
#elif target_enc600
				case XR1:
					reg.id = IXPIO_XR1;
					if (ioctl(fds[x].fd, IXPIO_READ_REG, &reg))
						puts("Failure of IXPIO_READ_REG:XR1\n");
					break;
				case XR2:
					reg.id = IXPIO_XR2;
					if (ioctl(fds[x].fd, IXPIO_READ_REG, &reg))
						puts("Failure of IXPIO_READ_REG:XR2\n");
					break;
				case XR3:
					reg.id = IXPIO_XR3;
					if (ioctl(fds[x].fd, IXPIO_READ_REG, &reg))
						puts("Failure of IXPIO_READ_REG:XR3\n");
					break;
				case XR4:
					reg.id = IXPIO_XR4;
					if (ioctl(fds[x].fd, IXPIO_READ_REG, &reg))
						puts("Failure of IXPIO_READ_REG:XR4\n");
					break;
				case XR5:
					reg.id = IXPIO_XR5;
					if (ioctl(fds[x].fd, IXPIO_READ_REG, &reg))
						puts("Failure of IXPIO_READ_REG:XR5\n");
					break;
				case XR6:
					reg.id = IXPIO_XR6;
					if (ioctl(fds[x].fd, IXPIO_READ_REG, &reg))
						puts("Failure of IXPIO_READ_REG:XR6\n");
					break;
				case DI1:
					reg.id = IXPIO_DI1;
					if (ioctl(fds[x].fd, IXPIO_READ_REG, &reg))
						puts("Failure of IXPIO_READ_REG:DI1\n");
					break;
				case DI2:
					reg.id = IXPIO_DI2;
					if (ioctl(fds[x].fd, IXPIO_READ_REG, &reg))
						puts("Failure of IXPIO_READ_REG:DI2\n");
					break;
				case DI3:
					reg.id = IXPIO_DI3;
					if (ioctl(fds[x].fd, IXPIO_READ_REG, &reg))
						puts("Failure of IXPIO_READ_REG:DI3\n");
					break;
				case DI4:
					reg.id = IXPIO_DI4;
					if (ioctl(fds[x].fd, IXPIO_READ_REG, &reg))
						puts("Failure of IXPIO_READ_REG:DI4\n");
					break;
				case DI5:
					reg.id = IXPIO_DI5;
					if (ioctl(fds[x].fd, IXPIO_READ_REG, &reg))
						puts("Failure of IXPIO_READ_REG:DI5\n");
					break;
				case DI6:
					reg.id = IXPIO_DI6;
					if (ioctl(fds[x].fd, IXPIO_READ_REG, &reg))
						puts("Failure of IXPIO_READ_REG:DI6\n");
					break;
#endif
				default:
					return FAILURE;
			}

			break;
		}
		++x;

	}
	return reg.value;

}

void myoutb(unsigned int base, unsigned char cdata)
{
	ixpio_reg_t reg;
	int x, match;

	reg.value = cdata;

	for (x = 0; x < MAX_CARDNO; x++){
		if((base >= fds[x].base) && (base < (fds[x].base + fds[x].iorange))){
			switch (base - fds[x].base){
				case TJ_CNTL:
					reg.id = IXPIO_TJ_CNTL;
					if (ioctl(fds[x].fd, IXPIO_WRITE_REG, &reg)){
						puts("Failure of IXPIO_WRITE_REG:TJ_CNTL");
					}
					break;

				case TJ_AUXC:
					reg.id = IXPIO_TJ_AUXC;
					if (ioctl(fds[x].fd, IXPIO_WRITE_REG, &reg)){
						puts("Failure of IXPIO_WRITE_REG:TJ_AUXC");
					}
					break;
					
				case DO:
					reg.id = IXPIO_DO;
					if (ioctl(fds[x].fd, IXPIO_WRITE_REG, &reg))
						puts("Failure of IXPIO_WRITE_REG");
					break;
					
#ifdef target_mstep3
				case FIFO1:
					reg.id = IXPIO_FIFO1;
					if (ioctl(fds[x].fd, IXPIO_WRITE_REG, &reg)){
						puts("Failure of IXPIO_WRITE_REG:FIFO1");
					}
					break;

				case RR:
					reg.id = IXPIO_RR;
					if (ioctl(fds[x].fd, IXPIO_WRITE_REG, &reg)){
						puts("Failure of IXPIO_WRITE_REG:RR");
					}
					break;
				
#elif target_enc600
				case XR1:
					reg.id = IXPIO_XR1;
					if (ioctl(fds[x].fd, IXPIO_WRITE_REG, &reg))
						puts("Failure of IXPIO_WRITE_REG:XR1");
					break;
					
				case XR2:
					reg.id = IXPIO_XR2;
					if (ioctl(fds[x].fd, IXPIO_WRITE_REG, &reg))
						puts("Failure of IXPIO_WRITE_REG:XR2");
					break;

				case XR3:
					reg.id = IXPIO_XR3;
					if (ioctl(fds[x].fd, IXPIO_WRITE_REG, &reg))
						puts("Failure of IXPIO_WRITE_REG:XR3");
					break;

				case XR4:
					reg.id = IXPIO_XR4;
					if (ioctl(fds[x].fd, IXPIO_WRITE_REG, &reg))
						puts("Failure of IXPIO_WRITE_REG:XR4");
					break;

				case XR5:
					reg.id = IXPIO_XR5;
					if (ioctl(fds[x].fd, IXPIO_WRITE_REG, &reg))
						puts("Failure of IXPIO_WRITE_REG:XR5");
					break;

				case XR6:
					reg.id = IXPIO_XR6;
					if (ioctl(fds[x].fd, IXPIO_WRITE_REG, &reg))
						puts("Failure of IXPIO_WRITE_REG:XR6");
					break;

#endif
				default:
					puts("FAILURE");
					break;
			}

			break;
		}
	}
}

void mymove(int x, int y)
{
	/* FIXME - gotoxy, the function of c++, not supported yet */
	printf("Left:%x, Top:%x\n", x, y);
}

void closeDev(){
	unsigned char j;
	for ( j = 0; j < MAX_CARDNO; j++) {
		close(fds[j].fd);
	}
}
