#include "piodio.h"

extern char device_module[LINE_SIZE + 1];
extern WORD channel_record;
extern WORD channel_init;
extern WORD gain_record;
extern WORD gain_init;

WORD PIODX6_PORT_ID[12] = {
                              IXPIO_P0, IXPIO_P1, IXPIO_P2,
                              IXPIO_P3, IXPIO_P4, IXPIO_P5,
                              IXPIO_P6, IXPIO_P7, IXPIO_P8,
                              IXPIO_P9, IXPIO_P10, IXPIO_P11
                          };

/* PIO-DA16 output data */
WORD PIODA16_Digital_Output(WORD fd, byte data)
{
  ixpio_digital_t dout;

  dout.data.u16 = data;

  if (ioctl(fd, IXPIO_DIGITAL_OUT, &dout))
  {
    close(fd);
    return PIODA_DIGITAL_OUTPUT_ERROR;
  }

  return SUCCESS;
}

/* PIO-DA16 input data */
WORD PIODA16_Digital_Input(WORD fd, WORD *di_data)
{
  ixpio_digital_t din;

  din.data.u16 = 0;

  if (ioctl(fd, IXPIO_DIGITAL_IN, &din))
  {
    close(fd);
    return PIODA_DIGITAL_INPUT_ERROR;
  }

  *di_data = din.data.u16;

  return SUCCESS;
}

WORD PIODA16_Int_Source_Init(WORD fd, WORD signal, WORD int_source)
{
  ixpio_reg_t reg;
  struct sigaction act_old;

  if (int_source != PIODA16_INT0 && int_source != PIODA16_INT1 && int_source != PIODA16_ALL_INT)
  {
    close(fd);
    sigaction(signal, &act_old, NULL);
    return PIODA_INT_SOURCE_DEFINED_ERROR;
  }

  /* configure board interrupt */
  reg.id = IXPIO_IMCR;
  reg.value = int_source;

  if (PIODA_OutputByte(fd, &reg))
  {
    sigaction(signal, &act_old, NULL);
    return PIODA_CONFIGURE_INTERRUPT_ERROR;
  }

  return SUCCESS;
}

// ******************************************************************
//   Function for Analog Output or Input cability
// ******************************************************************

/* PISO-813 Analog Input Voltage */
WORD PISO813_Analog_Input_Voltage(WORD fd, WORD channel, WORD gain, float *fvalue)
{
  ixpio_reg_t pgcr, mcsr, ai;
  
  /* PISO-813 Gain code setting */

  if ( gain_init || (!gain_init && gain_record != gain) ) 
  {
    pgcr.id = IXPIO_PGCR;
    pgcr.value = gain; 

    if ( PIODA_OutputByte(fd, &pgcr) ) return PIODA_SET_GAIN_ERROR;
 
    usleep(12); /* PGA + propagation settling delay */

    if ( gain_init ) gain_init = 0;

    gain_record = gain;
  }

  /* PISO-813 Input channel setting */
  if ( channel_init || (!channel_init && channel_record != channel) )
  {
    mcsr.id = IXPIO_MCSR;
    mcsr.value = channel; 
  
    if ( PIODA_OutputByte(fd, &mcsr) ) return PIODA_SET_CHANNEL_ERROR;
 
    usleep(10); /* multiplexer settling delay */

    if ( channel_init ) channel_init = 0;

    channel_record = gain;
  }

  ai.id = IXPIO_AD;

  if ( PIODA_InputByte(fd, &ai) ) return PIODA_ANALOG_INPUT_VOLTAGE_ERROR;
  *fvalue = (float)ai.value; 
 
  return SUCCESS;
}

