/* PIO/PISO Linux Static Libary.

   Author: Golden Wang

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

/* File level history (record changes for this file here.)

   Use vi or gvim to edit this file and set tabstop=4.

   v 0.2.0  7  Sep 2007 by Golden Wang
     give support to pisoa64/c64.
     give support to pisop32a32/p32c32.
     give support to pisop64.

   v 0.1.2  7  Sep 2007 by Golden Wang
     give support to piod96.

   v 0.1.1  7  Jul 2007 by Golden Wang
     give support to piod24/d56.

   v 0.1.0  11 Jun 2007 by Golde Wang
     give support to piod144/168.  

   v 0.0.0  24 May 2007 by Golden Wang
     create, blah blah.
    	
*/

/* the header that defined all variable and funtions */ 
#include "piodio.h"

/* defined private function here */
WORD PIODA_Init(void);
WORD PIODA_GetDeviceModuleName(void);
WORD PIODA_PortDir_Check(void);
WORD PIODA_ReadEEPROMAll(WORD);
WORD PIODA_EEP_READ(WORD, WORD, WORD *, WORD *);
WORD PIODA_COMM(WORD);
WORD PIODA_WR_BYTE(WORD, WORD);
WORD PIODA_RD_BYTE(WORD, WORD *);
//INLINE WORD PIODA_TEST(void);

//port_config_t *pconf;
dev_control_t *dev_ctl;
 
static float PIODA_fDeltaV[MAX_BOARD_NUMBER][16];
static float PIODA_fDeltaI[MAX_BOARD_NUMBER][16];
static WORD  PIODA_wN10V[MAX_BOARD_NUMBER][16];
static WORD  PIODA_wP10V[MAX_BOARD_NUMBER][16];
static WORD  PIODA_w00mA[MAX_BOARD_NUMBER][16];
static WORD  PIODA_w20mA[MAX_BOARD_NUMBER][16];
static WORD  PIODA_EEP;

char device_module[256];
char device_file[LINE_SIZE];
char ixpio[MAX_CARD_SUPPORT][LINE_SIZE];
WORD channel_record;
WORD channel_init = 1;
WORD gain_record;
WORD gain_init = 1;

/* open ixpio device file */
int PIODA_Open(char *dev_file)
{
  int fd;

  strcpy(device_file, dev_file); 
  fd = open(dev_file, O_RDWR);

  return fd;
}

/* close ixpio device file */
WORD PIODA_Close(WORD fd)
{
  /* free malloc memory */
  if ( dev_ctl ) free(dev_ctl);

  /* close device file */
  close(fd);
  return PIODA_NOERROR;
}

WORD PIODA_DriverInit(WORD fd)
{
  if(PIODA_GetDeviceModuleName()) return PIODA_MODULE_NAME_GET_ERROR;
  /* get EEPROM data */
  if (!strcmp(device_module,"ixpioda16"))
  {
    if(PIODA_ReadEEPROMAll(fd)) return PIODA_READ_EEPROM_ERROR;
  }
  PIODA_Init();

  return PIODA_NOERROR;
}

WORD PIODA_Init()
{
  dev_ctl = malloc(sizeof(dev_control_t));
  memset(dev_ctl, 0, sizeof(dev_control_t));
  if (!strcmp(device_module,"ixpiod144") || !strcmp(device_module,"ixpiod168"))
  {
    dev_ctl->portdircfs = PIOD144_168_PortDirCfs;
    dev_ctl->doutput_port = PIOD144_168_Digital_Output;
    dev_ctl->dinput_port = PIOD144_168_Digital_Input;
    dev_ctl->int_source_init = PIOD144_168_Int_Source_Init;
  }
  else if ( !strcmp(device_module,"ixpiod56") || !strcmp(device_module,"ixpiod24") )
  {
    dev_ctl->portdircfs = PIOD56_24_PortDirCfs;
    dev_ctl->doutput_port = PIOD56_24_Digital_Output;
    dev_ctl->dinput_port = PIOD56_24_Digital_Input;
    dev_ctl->int_source_init = PIOD56_24_Int_Source_Init;
  }
  else if ( !strcmp(device_module,"ixpiod96") )
  {
    dev_ctl->portdircfs = PIOD96_PortDirCfs;
    dev_ctl->doutput_port = PIOD96_Digital_Output;
    dev_ctl->dinput_port = PIOD96_Digital_Input;
    dev_ctl->int_source_init = PIOD96_Int_Source_Init;
  }
  else if ( !strcmp(device_module,"ixpisop8r8") )
  {
    dev_ctl->doutput = PISOP8R8_Digital_Output;
    dev_ctl->dinput = PISOP8R8_Digital_Input;
  }
  else if ( !strcmp(device_module,"ixpiso725") )
  {
    dev_ctl->doutput = PISO725_Digital_Output;
    dev_ctl->dinput = PISO725_Digital_Input;
    dev_ctl->int_source_init = PISO725_Int_Source_Init;
  }
  else if ( !strcmp(device_module,"ixpioda16") )
  {
    dev_ctl->doutput = PIODA16_Digital_Output;
    dev_ctl->dinput = PIODA16_Digital_Input;
    dev_ctl->int_source_init = PIODA16_Int_Source_Init;
  }
  else if ( !strcmp(device_module,"ixpisoa64") || !strcmp(device_module,"ixpisoc64") )
  {
    dev_ctl->doutput_port = PISOA64_C64_Digital_Output;
  }
  else if ( !strcmp(device_module,"ixpisop64") )
  {
    dev_ctl->dinput_port = PISOP64_Digital_Input;
  }
  else if (!strcmp(device_module,"ixpiso813"))
  {
    dev_ctl->ainput = PISO813_Analog_Input_Voltage;
  }
  else if (!strcmp(device_module,"ixpisop32c32") || !strcmp(device_module,"ixpisop32a32"))
  {
    dev_ctl->doutput_port = PISOP32A32_C32_Digital_Output;
    dev_ctl->dinput_port = PISOP32A32_C32_Digital_Input;
  }
  
  return PIODA_NOERROR;
}

/* get ixpio device module name */  
WORD PIODA_GetDeviceModuleName()
{
  FILE *fd_proc;
  char rbuf[254];
  char *module;
  int module_num = 0;
  int module_index = 0;

  fd_proc = fopen(IXPIO_PROC_FILE, "r");
 
  if (!fd_proc) return FAILURE;
 
  while (fgets(rbuf, 254, fd_proc))
  {
    if (!strncmp(rbuf, "mod:", 4))
    {
      module = strtok(strtok(rbuf + 4, "\n"), " ");

      while(module != NULL)
      {
        strcpy(ixpio[module_num],module);
        module = strtok(NULL," ");
        module_num++;
      }
    }

    if (!strncmp(rbuf, "dev:", 4))
    {
      if(strstr(rbuf,strtok(device_file + 4, "/")))
      {
        break;
      }
      
      module_index++;
    }
  }

  fclose(fd_proc);

  strcpy(device_module, ixpio[module_index]);
 
  return PIODA_NOERROR;
}

/* get ixpio device driver version that be defined in piodio.h */ 
WORD PIODA_GetDriverVersion(void)
{
  printf("Drive VERSION %s\n",IXPIO_DRIVER_VERSION);
  return PIODA_NOERROR;
}

WORD PIODA_GetLibaryVersion(void)
{
  printf("Libary VERSION %s\n",IXPIO_LIBARY_VERSION);
  return PIODA_NOERROR;
}

/* change port status */
WORD PIODA_PortDirCfs(WORD fd, WORD port, boolean io)
{
  WORD rs;

  /* port status are changed */   
  if ( !strcmp(device_module,"ixpiod144") )
  {
    if ( (short)port < 0 || (short)port > 17 ) return PIODA_PORT_DEFINED_ERROR;
  }
  else if ( !strcmp(device_module,"ixpiod168") )
  {
    if ( (short)port < 0 || (short)port > 20 ) return PIODA_PORT_DEFINED_ERROR;
  }
  else if ( !strcmp(device_module,"ixpiod96") )
  {
    if ( (short)port < 0 || (short)port > 11 ) return PIODA_PORT_DEFINED_ERROR;
  } 
  else if ( !strcmp(device_module,"ixpiod56") || !strcmp(device_module,"ixpiod24") )
  {
    if ( (short)port < 0 || (short)port > 2 ) return PIODA_PORT_DEFINED_ERROR;
  }

  rs = dev_ctl->portdircfs(fd, port, io);

  return rs;
}

/* output digital data */
WORD PIODA_Digital_Output(WORD fd, WORD port, byte data)
{
  WORD rs;
  if (!strcmp(device_module,"ixpiod144") || !strcmp(device_module,"ixpiod168") || !strcmp(device_module,"ixpiod96") || !strcmp(device_module,"ixpiod56") || !strcmp(device_module,"ixpiod24") )
  {
    rs = PIODA_PortDirCfs(fd, port, DIGITAL_OUTPUT);
    if ( rs ) return PIODA_DEVICE_DIO_INIT_ERROR;
    rs = dev_ctl->doutput_port(fd, port, data);
  }
  else
  {
    if ( !strcmp(device_module,"ixpisoa64") || !strcmp(device_module,"ixpisop32c32") || !strcmp(device_module,"ixpisop32a32") )
    {
      rs = dev_ctl->doutput_port(fd, port, data);
    }
    else
    {
      rs = dev_ctl->doutput(fd, data);
    }
  }

  return rs;
}

/* input digital data */
WORD PIODA_Digital_Input(WORD fd, WORD port, WORD *di_data)
{
  WORD rs;

  if ( !strcmp(device_module,"ixpiod144") || !strcmp(device_module,"ixpiod168") || !strcmp(device_module,"ixpiod96") || !strcmp(device_module,"ixpiod56") || !strcmp(device_module,"ixpiod24") )
  {
    rs = PIODA_PortDirCfs(fd, port, DIGITAL_INPUT);
    if ( rs ) return PIODA_DEVICE_DIO_INIT_ERROR;
    rs = dev_ctl->dinput_port(fd, port, di_data);
  }
  else
  {
    if ( !strcmp(device_module,"ixpisop64") || !strcmp(device_module,"ixpisop32c32") || !strcmp(device_module,"ixpisop32a32") )
    {
      rs = dev_ctl->dinput_port(fd, port, di_data);
    }
    else
    {
      rs = dev_ctl->dinput(fd, di_data);
    }
  }

  return rs;
}


// ******************************************************************
// Analog-Output for Voltage with calibration
//
// Parameters :
//   wBoardNo  :[in] Board Number, Range is 0 to 7.
//   wChannels :[in] Analog-Output to which channel ?
//                   0 to 15 for PIO-DA16
//                   0 to  7 for PIO-DA8
//                   0 to  3 for PIO-DA4
//   fvalue    :[in] -10v to +10v
// ******************************************************************
WORD PIODA_AnalogOutputCalVoltage(WORD fd, WORD channel, float fvalue)
{
  WORD data;
  WORD wBase;
  float fTmp;

  fTmp = PIODA_fDeltaV[fd][channel];

  if ( fTmp != 0 )
  {
    data = (WORD)( (fvalue+10.0F)/fTmp  + PIODA_wN10V[fd][channel] );
  } 
  else
  {
    return PIODA_EEPROM_DATA_ERROR;
  }

  ixpio_analog_t ao;

  ao.channel = channel;  
  ao.data.u16 = data;

  if (PIODA_OutputAnalog(fd, &ao)) return PIODA_OUTPUT_CALVOLTAGE_ERROR;
  
  return PIODA_NOERROR;
}

// ******************************************************************
// Analog-Output for Voltage without calibration
//
// Parameters :
//   wBoardNo  :[in] Board Number, Range is 0 to 7.
//   wChannels :[in] Analog-Output to which channel ?
//                   0 to 15 for PIO-DA16
//                   0 to  7 for PIO-DA8
//                   0 to  3 for PIO-DA4
//   fvalue    :[in] -10v to +10v
// ******************************************************************
WORD PIODA_AnalogOutputVoltage(WORD fd, WORD channel, float fvalue)
{
  WORD data;
  float fStep;

  fStep=16383.0F/20.0F;

  data= (WORD)(fvalue*fStep+8192.0F);

  ixpio_analog_t ao;
  ao.channel = channel;
  ao.data.u16 = data;

  if (PIODA_OutputAnalog(fd, &ao)) return PIODA_OUTPUT_VOLTAGE_ERROR;  

  return PIODA_NOERROR;
}

// ******************************************************************
// Analog-Output for Current with calibration
//
// Parameters :
//   wBoardNo  :[in] Board Number, Range is 0 to 7.
//   wChannels :[in] Analog-Output to which channel ?
//                   0 to 15 for PIO-DA16
//                   0 to  7 for PIO-DA8
//                   0 to  3 for PIO-DA4
//   fvalue    :[in] 0mA to +20mA
// ******************************************************************
WORD PIODA_AnalogOutputCalCurrent(WORD fd, WORD channel, float fvalue)
{
  WORD data;
  float fTmp;

  fTmp = PIODA_fDeltaI[fd][channel];

  if ( fTmp != 0 )
  {
    data = (WORD)( fvalue/fTmp  + PIODA_w00mA[fd][channel] );
  }
  else
  {
    return PIODA_EEPROM_DATA_ERROR;
  }

  ixpio_analog_t ao;

  ao.channel = channel; 
  ao.data.u16 = data;

  if (PIODA_OutputAnalog(fd, &ao)) return PIODA_OUTPUT_CALCURRENT_ERROR;
  
  return PIODA_NOERROR;
}

// ******************************************************************
// Analog-Output for Current without calibration
//
// Parameters :
//   wBoardNo  :[in] Board Number, Range is 0 to 7.
//   wChannels :[in] Analog-Output to which channel ?
//                   0 to 15 for PIO-DA16
//                   0 to  7 for PIO-DA8
//                   0 to  3 for PIO-DA4
//   fvalue    :[in] 0mA to +20mA
// ******************************************************************
WORD PIODA_AnalogOutputCurrent(WORD fd, WORD channel, float fvalue)
{
  WORD data;
  float fStep;

  fStep=8192.0F/20.0F;

  data= (WORD)(fvalue*fStep+8191.0F);

  ixpio_analog_t ao;
  ao.channel = channel;
  ao.data.u16 = data;

  if (PIODA_OutputAnalog(fd, &ao)) return PIODA_OUTPUT_CURRENT_ERROR;

  return PIODA_NOERROR;
}

WORD PIODA_AnalogInputVoltage(WORD fd, WORD channel, WORD gain, float *fvalue)
{
  WORD rs;
 
  rs = dev_ctl->ainput(fd, channel, gain, fvalue);

  return rs;
}

/* support some ixpio card interrupt function */
WORD PIODA_IntInstall(WORD fd, HANDLE hisr, WORD signal, WORD int_source, WORD activemode)
{
  WORD rs;
  ixpio_signal_t sig;
  struct sigaction act,act_old;

  act.sa_handler = hisr;
  sigemptyset(&act.sa_mask);
  sigaddset(&act.sa_mask, signal);

  if(sigaction(signal, &act, &act_old))
  {
    close(fd);
    return FAILURE;
  }

  /* init I/O card INT and check int_source is right or not */
  rs = dev_ctl->int_source_init(fd, signal, int_source);

  if ( rs ) return rs;	/* Int init error */	

  /* signal condiction */
  sig.sid = signal;
  sig.pid = getpid();
  sig.is = int_source;            /* signaling for the channels */
 
  if (activemode == 2 )
  {
    sig.bedge = int_source;             /* signaling for both edges */
  }
  else if (activemode == 1)
  {
    
    sig.edge = int_source;        /* signaling for positive edges */
    sig.bedge = 0;                /* don't signal for both edges */
  }
  else if (activemode == 0)
  {
    sig.edge = 0;        /* signaling for negative edges */
    sig.bedge = 0;
  }
  else
  {
    close(fd);
    sigaction(signal, &act_old, NULL);
    return PIODA_ACTIVEMODE_DEFINED_ERROR;
  }

  if (ioctl(fd, IXPIO_SIG, &sig))
  {
    close(fd); 
    sigaction(signal, &act_old, NULL);
    return PIODA_ADD_SIGNAL_ERROR;
  }

  return PIODA_NOERROR;
}

/* close interrupt function */
WORD PIODA_IntRemove(WORD fd, WORD sig_id)
{
  ixpio_reg_t reg;
  struct sigaction act_old;
  
  sigaction(sig_id, &act_old, NULL);

  /* disable all board interrupt */
  reg.id = IXPIO_IMCR;
  reg.value = 0;

  if (PIODA_OutputByte(fd, &reg)) return PIODA_CONFIGURE_INTERRUPT_ERROR;

  return PIODA_NOERROR;
}

WORD PIODA_ReadEEPROMAll(WORD fd)
{
  WORD i;
  WORD bHi, bLo;
  WORD wTmp;
  WORD wCh;

  ixpio_reg_t reg;

  /* AUX 4/3/2 are DO, others DI */
  reg.value = 0x1c;
  reg.id = IXPIO_ACR;

  if (PIODA_OutputByte(fd, &reg)) return PIODA_AUX_CONTROL_ERROR;

  /* ALL DO are low */
  reg.value = 0x00;
  reg.id = IXPIO_ADR;

  if (PIODA_OutputByte(fd, &reg)) return PIODA_AUX_DATA_ERROR;

  //************************************************************
  // Read calibration data from EEPROM
  //************************************************************

  for(i=0; i<64; i++)
  {
     if(PIODA_EEP_READ(fd,(WORD)(i),&bHi,&bLo)) return PIODA_READ_EEPROM_ERROR;

     if( i<16 )
     {
        PIODA_wN10V[fd][i]=(WORD)(bHi*256 + bLo);
     }
     else if( (i>=16) && (i<32) )
     {
        PIODA_wP10V[fd][i-16]=(WORD)(bHi*256 + bLo);
     }
     else if( (i>=32) && (i<48) )
     {
        PIODA_w00mA[fd][i-32]=(WORD)(bHi*256 + bLo);
     }
     else if( i>=48 )
     {
        PIODA_w20mA[fd][i-48]=(WORD)(bHi*256 + bLo);
     }
  }

  for(i=0; i<16; i++)
  {
    wTmp = (WORD)((float)PIODA_wP10V[fd][i]-(float)PIODA_wN10V[fd][i]);

    //printf("N10V %d : %u\n", i, PIODA_wN10V[fd][i]);
    //printf("P10V %d : %u\n", i, PIODA_wP10V[fd][i]);
    //printf("Tmp V %d : %u\n", i, wTmp);
   
    if ( wTmp > 0 )
    {
      PIODA_fDeltaV[fd][i]=(float)(20.0/(float)wTmp);
    }
    else
    {
      PIODA_fDeltaV[fd][i]=0;
    }
    //printf("DV %d : %f\n", i, PIODA_fDeltaV[fd][i]);
    //printf("\n");

    wTmp = (WORD)((float)PIODA_w20mA[fd][i]-(float)PIODA_w00mA[fd][i]);

    if ( wTmp > 0 )
    {
      PIODA_fDeltaI[fd][i]=(float)(20.0/(float)wTmp);
    }
    else
    {
      PIODA_fDeltaI[fd][i]=0;
    }
    //printf("W00MA %d : %u\n", i, PIODA_w00mA[fd][i]);
    //printf("W20MA %d : %u\n", i, PIODA_w20mA[fd][i]);
    //printf("wTmp I %d : %u\n", i, wTmp);
    //printf("DI %d : %f\n", i, PIODA_fDeltaI[fd][i]);
    //printf("\n");
  }
 
  return PIODA_NOERROR;
}

WORD PIODA_EEP_READ(WORD fd, WORD wOffset, WORD *bHi, WORD *bLo)
{
  WORD wBase, wHi, wLo;
  ixpio_reg_t reg;

  if (PIODA_COMM(fd)) return FAILURE;

  // read command & address
  wOffset=(WORD)(wOffset|0x80);
  if(PIODA_WR_BYTE(fd,wOffset)) return FAILURE;

  PIODA_EEP=(WORD)(PIODA_EEP&0x0c);
  reg.id = IXPIO_ADR;
  reg.value = PIODA_EEP;

  if (PIODA_OutputByte(fd, &reg)) return FAILURE;

  PIODA_EEP=(WORD)(PIODA_EEP&0x14);
  reg.value = PIODA_EEP;

  if (PIODA_OutputByte(fd, &reg)) return FAILURE;

  PIODA_EEP=(WORD)(PIODA_EEP|0x08);
  reg.value = PIODA_EEP;

  if (PIODA_OutputByte(fd, &reg)) return FAILURE;

  PIODA_EEP=(WORD)(PIODA_EEP&0x14);
  reg.value = PIODA_EEP;

  if (PIODA_OutputByte(fd, &reg)) return FAILURE;

  PIODA_RD_BYTE(fd, &wHi);
      *bHi = (byte) wHi;
  // read low  8-bit data
  PIODA_RD_BYTE(fd, &wLo);
      *bLo = (byte) wLo;

  PIODA_EEP=(WORD)(PIODA_EEP&0x14);
  reg.value = PIODA_EEP;

  if (PIODA_OutputByte(fd, &reg)) return FAILURE;

  PIODA_EEP=(WORD)(PIODA_EEP&0x18);
  reg.value = PIODA_EEP;

  if (PIODA_OutputByte(fd, &reg)) return FAILURE;

  return PIODA_NOERROR;
}

// send start 2-bit (0,1)
//-----------------------------------------------
// AUX2=D/O=CS of EEP          AUX3=D/O=SK of EEP
// AUX4=D/O=DI of EEP          AUX5=D/I=DO of EEP
//-----------------------------------------------
WORD PIODA_COMM(WORD fd)
{
  ixpio_reg_t reg;

  PIODA_EEP=0x04;
  reg.value = PIODA_EEP;
  reg.id = IXPIO_ADR;
  if (PIODA_OutputByte(fd, &reg)) return FAILURE;

  PIODA_EEP=0x0c;
  reg.value = PIODA_EEP;

  if (PIODA_OutputByte(fd, &reg)) return FAILURE;

  PIODA_EEP=0x04;
  reg.value = PIODA_EEP;
  if (PIODA_OutputByte(fd, &reg)) return FAILURE;

  PIODA_EEP=0x14;
  reg.value = PIODA_EEP;
  if (PIODA_OutputByte(fd, &reg)) return FAILURE;

  PIODA_EEP=0x1c;
  reg.value = PIODA_EEP;

  if (PIODA_OutputByte(fd, &reg)) return FAILURE;

  return PIODA_NOERROR;
}

WORD PIODA_WR_BYTE(WORD fd, WORD bData)
{
  WORD i,j;
  ixpio_reg_t reg;

  for(i=0; i<8; i++)
  {
    PIODA_EEP=(WORD)(PIODA_EEP&0x14);
    reg.value = PIODA_EEP;
    reg.id = IXPIO_ADR;

    if (PIODA_OutputByte(fd, &reg)) return FAILURE;

    j=(WORD)(bData&0x80);
    bData=(WORD)(bData<<1);

    if( j==0 )         // DI=0
    {
      PIODA_EEP=(WORD)(PIODA_EEP&0x0c);
      reg.value = PIODA_EEP;
       if (PIODA_OutputByte(fd, &reg)) return FAILURE;
    }
    else               // DI=1
    {
      PIODA_EEP=(WORD)(PIODA_EEP|0x10);
      reg.value = PIODA_EEP;

      if (PIODA_OutputByte(fd, &reg)) return FAILURE;
    }

    PIODA_EEP=(WORD)(PIODA_EEP|0x08);
    reg.value = PIODA_EEP;

    if (PIODA_OutputByte(fd, &reg)) return FAILURE;
  }

  return PIODA_NOERROR; 
}

// read the next 8-bit
WORD PIODA_RD_BYTE(WORD fd, WORD *bData)
{
  byte i,j,k,Val;
  ixpio_reg_t reg, reg2;
  Val=0x80;
  k=0;

  for(i=0; i<8; i++)
  {
    PIODA_EEP=(WORD)(PIODA_EEP&0x14);
    reg.value = PIODA_EEP;
    reg.id = IXPIO_ADR;

    if (PIODA_OutputByte(fd, &reg)) return FAILURE;

    reg2.id = IXPIO_APSR;
    if (PIODA_InputByte(fd, &reg2)) return FAILURE;

    j=(byte)((reg2.value)&0x20);
    if( j!=0 ) k+=Val;     // DO=1

    PIODA_EEP=(WORD)(PIODA_EEP|0x08);
    reg.value = PIODA_EEP;
    if (PIODA_OutputByte(fd, &reg)) return FAILURE;

    Val=(byte)(Val>>1);
  }

  *bData = k;

  return PIODA_NOERROR;
}

WORD PIODA_OutputAnalog(WORD fd, ixpio_analog_t *p_ao)
{
  if (ioctl(fd, IXPIO_ANALOG_OUT, p_ao))
  {
    close(fd);
    return FAILURE;
  }

  return PIODA_NOERROR;
}

WORD PIODA_OutputByte(WORD fd, ixpio_reg_t *p_reg)
{
   if (ioctl(fd, IXPIO_WRITE_REG, p_reg))
   {
     close(fd);
     return FAILURE;
   }

   return PIODA_NOERROR;
}

WORD PIODA_InputByte(WORD fd, ixpio_reg_t *p_reg)
{
  if (ioctl(fd, IXPIO_READ_REG, p_reg))
  {
    close(fd);
    return FAILURE;
  }

  return PIODA_NOERROR;
}

